"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.envPlugin = exports.defaultEnv = exports.lastArg = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _nodeFetch = _interopRequireDefault(require("node-fetch"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// import { OBJ } from './utils';
const lastArg = R.pipe(Array.of, R.last);
exports.lastArg = lastArg;
const defaultEnv = {
  // '>': (a, b) => a > b,
  // '<=': (a, b) => a <= b,
  // '>=': (a, b) => a >= b,
  // '==': (a, b) => Object.is(a, b),
  // '!=': (a, b) => !Object.is(a, b),
  // '===': (a, b) => a === b,
  // '!==': (a, b) => a !== b,
  // get,
  undefined,
  Array,
  Object,
  String,
  Number,
  Promise,
  newPromise: (...args) => new Promise(...args),
  Date,
  newDate: (...args) => new Date(...args),
  Math,
  parseInt,
  parseFloat,
  Set,
  Map,
  newSet: (...args) => new Set(...args),
  newMap: (...args) => new Map(...args),
  RegExp,
  fetch: _nodeFetch.default
};
exports.defaultEnv = defaultEnv;

const envPlugin = () => ({
  staticEnv: {
    console,
    lastArg,
    ',': lastArg,
    of: Array.of,
    void: () => undefined // noop: () => undefined,

  },
  onLeave: ({
    state,
    current,
    envHas,
    getFromEnv
  }) => {
    if (state !== 'untouched') return undefined;

    if (R.is(String, current) && envHas(current)) {
      // return envHas(current) ? getFromEnv(current) : undefined;
      // console.log('ENV:', current, getFromEnv(current));
      return {
        type: 'REPLACE',
        current: getFromEnv(current)
      };
    }

    return undefined;
  }
});

exports.envPlugin = envPlugin;
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.evaluatePlugin = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _triFp = require("tri-fp");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// import { OBJ } from './utils';
// const unProcessedTypes = ['Boolean', 'Number', 'Null', 'Undefined', 'Function', 'RegExp'];
// const isUnProcessedType = R.pipe(R.type, R.includes(R.__, unProcessedTypes));
const shouldEvaluateString = R.both(R.startsWith('`'), R.endsWith('`'));
const extractStr = R.slice(1, -1);

const evaluatePlugin = () => ({
  onLeave: ({
    state,
    current
  }) => {
    if (state !== 'untouched') return undefined;

    if (R.is(String, current)) {
      const isStrConst = shouldEvaluateString(current);
      return isStrConst ? {
        type: 'REPLACE',
        current: extractStr(current)
      } : undefined; // return { type: 'REPLACE', current: envHas(current) ? getFromEnv(current) : undefined };
    }

    if (Array.isArray(current)) {
      const [fn, ...args] = current; // console.log('Evaluate Array:', fn, typeof fn);

      if (!(typeof fn === 'function')) return {
        type: 'ERROR',
        error: Error(`Not a function: ${fn}`)
      };
      const [error, result] = (0, _triFp.bi)(fn)(...args);
      if (error) return {
        type: 'ERROR',
        error
      };
      return {
        type: 'REPLACE',
        current: result
      };
    }

    return undefined;
  }
});

exports.evaluatePlugin = evaluatePlugin;
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createRunner = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _rambdax = require("rambdax");

var _lodash = _interopRequireDefault(require("lodash.isplainobject"));

var _utils = require("./utils");

var _env = require("./env");

var _evaluate = require("./evaluate");

var _vars = require("./vars");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// import set from 'lodash.set';
// import deepFreeze from 'deep-freeze-strict';
const pluginEnvs = R.pipe(R.pluck('staticEnv'), R.mergeAll);
const promiseWrapperKey = '%_PROMISE_OBJECT_%';
const maybeWrap = R.when(R.is(Promise), R.objOf(promiseWrapperKey));

const maybeUnwrap = node => {
  if (!(0, _lodash.default)(node)) return node;
  const keys = Object.keys(node);
  return keys.length === 1 && keys[0] === promiseWrapperKey ? node[promiseWrapperKey] : node;
};

const UNTOUCHED = 'untouched';
const REPLACED = 'replaced';
const ERROR = 'error';

const pluginCallbackReducer = ctx => ({
  state,
  current,
  error
}, fn) => {
  if (state === ERROR) return {
    state,
    error
  };
  const iterObj = fn({ ...ctx,
    state,
    current
  });
  if ((iterObj === null || iterObj === void 0 ? void 0 : iterObj.type) === 'ERROR') return {
    state: ERROR,
    error: iterObj === null || iterObj === void 0 ? void 0 : iterObj.error
  };
  if ((iterObj === null || iterObj === void 0 ? void 0 : iterObj.type) === 'REPLACE') return {
    state: REPLACED,
    current: iterObj === null || iterObj === void 0 ? void 0 : iterObj.current
  };
  return {
    state,
    current
  };
};

const createRunner = ({
  staticEnv = _utils.OBJ,
  // scopes = OBJ,
  plugins = [(0, _env.envPlugin)(), (0, _evaluate.evaluatePlugin)(), (0, _vars.varsPlugin)()]
} = _utils.OBJ) => {
  const env = { ...pluginEnvs(plugins),
    ...staticEnv
  }; // TODO: deepFreeze?

  const envHas = pth => (0, _rambdax.hasPath)(pth, env);

  const getFromEnv = (0, _utils.getPath)(env);
  const onEnterFns = R.pipe(R.pluck('onEnter'), R.filter(R.is(Function)))(plugins);
  const onLeaveFns = R.pipe(R.pluck('onLeave'), R.filter(R.is(Function)))(plugins);

  const processNode = async originalNode => {
    // TODO: freeze originalNode
    // ENTER
    const onEnterCtx = R.reduce(pluginCallbackReducer({
      originalNode,
      envHas,
      getFromEnv
    }), {
      state: UNTOUCHED,
      current: originalNode
    })(onEnterFns);
    if (onEnterCtx.state === ERROR) return (0, _utils.ensureError)(onEnterCtx.error);
    /* eslint-disable fp/no-let, fp/no-mutation */

    let node = onEnterCtx.current; // const node = originalNode;
    // RECURSE

    if (Array.isArray(originalNode) || (0, _lodash.default)(originalNode)) {
      node = await (0, _rambdax.mapAsync)(processNode, node);
      node = R.map(maybeUnwrap)(node);
    }

    if (R.is(Error, node)) return node; // PARALLEL AWAIT POSSIBLE PROMISES

    if ((0, _lodash.default)(node)) node = await (0, _utils.parallelAwaitObjectVals)(node);
    /* eslint-enable fp/no-let, fp/no-mutation */
    // if (R.is(Error, halfEvaluated)) return halfEvaluated; // CAN THIS EVER HAPPEN?!
    // console.log('halfEvaluated:', halfEvaluated);
    // LEAVE

    const onLeaveCtx = R.reduce(pluginCallbackReducer({
      originalNode,
      node,
      envHas,
      getFromEnv
    }), {
      state: onEnterCtx.state,
      current: node
    })(onLeaveFns);
    const unfoundIdentifier = R.is(String, onLeaveCtx.current) && onLeaveCtx.state === UNTOUCHED;
    if (onLeaveCtx.state === ERROR) return (0, _utils.ensureError)(onLeaveCtx.error);
    if (unfoundIdentifier) return Error(`Unfound identifier: ${onLeaveCtx.current}`);
    return maybeWrap(onLeaveCtx.current);
  };

  return R.pipe(processNode, maybeUnwrap);
};

exports.createRunner = createRunner;
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parallelAwaitObjectVals = exports.ensureError = exports.getPath = exports.isForbidden = exports.forbiddenIdentifiers = exports.FUNC = exports.ARR = exports.OBJ = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _rambdax = require("rambdax");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// import isPlainObject from 'lodash.isplainobject';
const OBJ = Object.freeze(Object.create(null));
exports.OBJ = OBJ;
const ARR = Object.freeze([]);
exports.ARR = ARR;
const FUNC = Function.prototype;
exports.FUNC = FUNC;
const forbiddenIdentifiers = ['eval', 'Function', 'constructor', 'setTimeout', 'setInterval'];
exports.forbiddenIdentifiers = forbiddenIdentifiers;
const isForbidden = R.pipe(R.when(R.is(String), R.split('.')), R.anyPass(R.map(R.includes)(forbiddenIdentifiers)));
exports.isForbidden = isForbidden;

const getPath = obj => pth => isForbidden(pth) ? Error(`${pth} is forbidden`) : (0, _rambdax.path)(pth, obj);

exports.getPath = getPath;

const ensureError = error => R.is(Error, error) ? error : R.is(String, error) ? Error(error) : Error('Unknown error from plugin');

exports.ensureError = ensureError;

const parallelAwaitObjectVals = async obj => {
  const orderedPairs = R.toPairs(obj);
  const keys = R.pluck(0)(orderedPairs);
  const vals = R.pluck(1)(orderedPairs);
  const resolvedValues = await Promise.all(vals);
  return R.zipObj(keys, resolvedValues);
};

exports.parallelAwaitObjectVals = parallelAwaitObjectVals;
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.varsPlugin = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _lodash = _interopRequireDefault(require("lodash.set"));

var _lodash2 = _interopRequireDefault(require("lodash.isplainobject"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// import { assocPath } from 'rambdax';
// import get from 'lodash.get';
const checkAndOmitPrefix = (prefix = '') => path => {
  const isVar = R.startsWith(prefix, path);
  return [isVar ? R.drop(prefix.length, path) : path, isVar];
};

const varsPlugin = ({
  prefix = '$',
  vars = {}
} = _utils.OBJ) => {
  const getVarPath = checkAndOmitPrefix(prefix);
  const getVar = R.when(R.complement(R.is(Error)), (0, _utils.getPath)(vars)); // const hasVar = TODO: implement and use in onLeave return

  const setVar = R.curry((pth, val) => {
    const res = (0, _utils.getPath)((0, _lodash.default)(vars, pth, val))(pth); // console.log('setVar:', res, pth, val, vars);

    return res;
  });

  const setVarFlipped = (val, key) => setVar(key, val);

  return {
    staticEnv: {
      getVar,
      setVar
    },
    onEnter: ({
      current,
      state
    }) => {
      if (state !== 'untouched') return undefined;

      if (R.is(String, current)) {
        const [path, isVar] = getVarPath(current); // console.log('VAR:', current, vars, path, getVar(path));

        return isVar ? {
          type: 'REPLACE',
          current: getVar(path)
        } : undefined; // return { type: 'REPLACE', current: isVar ? getVar(path) : undefined };
      }

      return undefined;
    },
    onLeave: ({
      current
    }) => {
      // console.log('varsPlugin onLeave:', node, resolvedNode, altered);
      // TODO: If found in vars but altered -- still return vars version?
      if ((0, _lodash2.default)(current)) R.forEachObjIndexed(setVarFlipped, current); // console.log(vars);

      return undefined;
    }
  };
};

exports.varsPlugin = varsPlugin;
