import * as R from 'ramda';
import { path } from 'rambdax';
// import isPlainObject from 'lodash.isplainobject';

export const OBJ = Object.freeze(Object.create(null));
export const ARR = Object.freeze([]);
export const FUNC = Function.prototype;

export const forbiddenIdentifiers = [
  'eval',
  'Function',
  'constructor',
  'setTimeout',
  'setInterval',
];
export const isForbidden = R.pipe(
  R.when(R.is(String), R.split('.')),
  R.anyPass(R.map(R.includes)(forbiddenIdentifiers)),
);
export const getPath = (obj) => (pth) =>
  isForbidden(pth) ? Error(`${pth} is forbidden`) : path(pth, obj);

export const ensureError = (error) =>
  R.is(Error, error)
    ? error
    : R.is(String, error)
    ? Error(error)
    : Error('Unknown error from plugin');

export const parallelAwaitObjectVals = async (obj) => {
  const orderedPairs = R.toPairs(obj);
  const keys = R.pluck(0)(orderedPairs);
  const vals = R.pluck(1)(orderedPairs);
  const resolvedValues = await Promise.all(vals);
  return R.zipObj(keys, resolvedValues);
};
